
// DoodlePanel.java
// Andrew Davison, July 2013, ad@fivedots.coe.psu.ac.th

/* A panel that for drawing colored dots. It also creates the
   Leap controller and listener.

   The dots are updated by the listener calling:
      * addPoint()
      * movePoint()
      * cycleColor()
      * undo()

   The listener is stopped by the top-level LeapDoodle calling:
      * stopListening()
*/

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;

import com.leapmotion.leap.*;



public class DoodlePanel extends JPanel
{
  private static final int MAXPOINTS = 5000;

  // dimensions of the panel, which are fixed
  private static final int PWIDTH = 800;   
  private static final int PHEIGHT = 500;


  // for storing the drawable dots and their colors
  private Point[] points = new Point[MAXPOINTS];
  private Color[] pointsColor = new Color[MAXPOINTS];
  private int nPoints = 0;   // index into the two arrays

  // range of colors that can be cycled through
  private Color[] colors = {Color.BLUE, Color.GREEN, Color.RED, Color.BLACK, Color.GRAY};
  private int currColor = 0;   // starting color is blue

  private DoodleListener listener;
  private Controller controller;


  public DoodlePanel()
  {
    setBackground(Color.WHITE);

    // the first dot starts in the center of the panel, and is blue
    points[nPoints] = new Point(PWIDTH/2, PHEIGHT/2);
    pointsColor[nPoints] = colors[currColor];
    nPoints++;

    listener = new DoodleListener(this);
    controller = new Controller();
    controller.addListener(listener);
  } // end of DoodlePanel() constructor


  public Dimension getPreferredSize()
  {   return new Dimension(PWIDTH, PHEIGHT); }



  public void paintComponent(Graphics g)
  // repaint the panel by redrawing all the colored dots
  { 
    super.paintComponent(g);   // repaint standard stuff first
    Graphics2D g2 = (Graphics2D) g;

    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                        RenderingHints.VALUE_ANTIALIAS_ON);  // so dots are round

    for (int i=0; i < nPoints; i++) {
      g2.setColor( pointsColor[i]);
      g2.fillOval( points[i].x, points[i].y, 12, 12); 
    }
  }



  public void addPoint(Point2D.Float normPt)
  /* Record a dot at the specified position, then request a repaint.
     The normPt point is an offset, with
     the x- and y- values in the range -1 to 1 */
  // called by the listener
  {  
    if (normPt == null)
      return;
    if ((normPt.x == 0) && (normPt.y == 0))
      return;      // do nothing since offset is zero

    if (nPoints < MAXPOINTS) {    // add a new dot
      points[nPoints] = toScrCoords(normPt.x, normPt.y);
      pointsColor[nPoints] = colors[currColor];
      nPoints++;
      repaint();   // the repaint will call paintComponent()
    }
  }  // end of addPoint()


  private Point toScrCoords(float xNorm, float yNorm)
  /* The norm values are offsets, in the range -1 to 1
     Add the offset (multiplied by 12 to make it bigger) to the
     previous point's position to get a new (x, y) position.
     Don't allow the new dot to be located outside the panel.
  */
  {
    int x = (int)Math.round( (float)points[nPoints-1].x + xNorm*12);
    int y = (int)Math.round( (float)points[nPoints-1].y + yNorm*12);

    // restrict the new dot to be visible on the panel
    if (x < 0)
      x = 0;
    else if (x > PWIDTH-1)
      x = PWIDTH-1;
    
    if (y < 0)
      y = 0;
    else if (y > PHEIGHT-1)
      y = PHEIGHT-1;
    return new Point(x,y);
  }  // end of toScrCoords()



  public void movePoint(Point2D.Float normPt)
  /* Move the current dot by calculating a new (x,y) position using
     toScrCoords(), but assign that value to the current dot rather
     than create a new one. Then request a repaint. */
  // called by the listener
  {  
    if (normPt == null)
      return;
    if ((normPt.x == 0) && (normPt.y == 0))
      return;     // do nothing since offset is zero

    points[nPoints-1] = toScrCoords(normPt.x, normPt.y);  // change current dot
    repaint();
  }  // end of movePoint()



  public void cycleColor(boolean isClockWise)
  /* cycle through the colors by incrementing (or decrementing) 
     the currColor integer, modulo the size of colors[].
     Then request a repaint. */
  // called by the listener
  {
    if (isClockWise)    // increment
      currColor = (currColor + 1) % colors.length;
    else {   // decrement
      currColor--;
      if (currColor < 0)
        currColor = colors.length-1;
    }
    pointsColor[nPoints-1] = colors[currColor];
    repaint();
  }  // end of cycleColor()



  public void undo()
  /* Delete a point by decrementing the nPoints value which indexes into
     the points[] and pointsColor[] arrays. However, do not delete the
     first point, since this is needed to calculate the second point
     by adding an offset to it. Then request a repaint. */
  // called by the listener
  {  
    if (nPoints == 1)    // don't delete the first point
      return;
    nPoints--;
    repaint();
  }  // end of undo()




  public void stopListening()
  // remove the listener;
  // called from top-level at termination
  {  controller.removeListener(listener);  }


} // end of DoodlePanel class
