
// ModelBehavior.java
// Andrew Davison, July 2013, ad@fivedots.coe.psu.ac.th

/* This behavior is time-driven, so that it 
   update the sprite's position/rotation every DELAY ms.
   It directly accesses the controller's current frame, so
   does not require a subclass of LeapMotion SDK's Listener class.

   Use Leap's palm position to move the sprite, and a circle gesture
   to rotate it. Translation is restricted to the XZ plane, and 
   rotation is only around the y-axis.

   The Sprite3D methods called from here:
     * moveBy(x-step, y-step)
     * doRotateY(radian-step)   // rotation around y-axis of 3D world
*/

import java.awt.event.*;
import java.awt.AWTEvent;
import java.util.Enumeration;

import javax.media.j3d.*;
import javax.vecmath.*;

import com.leapmotion.leap.*;
import com.leapmotion.leap.Gesture.*;



public class ModelBehavior extends Behavior
{
  private final static int DELAY = 100;   // ms, time between updates

  // ignore x- z- distances smaller than these constants
  private final static float MIN_X_DIST = 40;   // mm
  private final static float MIN_Z_DIST = 30;   // mm

  private final static double MOVE_AMT = 0.3;                // translation step
  private final static double ROTATE_AMT = Math.PI / 16.0;   // rotation step


  private Sprite3D sprite;   // the 3D sprite being controlled from here
  private Controller controller;
  private WakeupCondition timeOut;   
                            // time-based condition to wakeup this behavior



  public ModelBehavior(Sprite3D s)
  {
    sprite = s;

    controller = new Controller();    // no listener used
    controller.enableGesture(Gesture.Type.TYPE_CIRCLE);
                
    timeOut = new WakeupOnElapsedTime(DELAY);
  } // end of ModelBehavior()



  public void initialize( )
  {  wakeupOn(timeOut);  }


  public void processStimulus(Enumeration criteria)
  // update the sprite every DELAY milliseconds
  {
    // ignore Java 3D criteria; look at Controller's current frame instead
    if(controller.isConnected()) {
      Frame frame = controller.frame();

      // translate the model using the user's palm x- and z- position
      if (!frame.hands().empty()) {
        Hand hand = frame.hands().get(0);          // access the first hand 
        Vector handPos = hand.palmPosition();
        // System.out.println("Palm: " + handPos);
        translateModel(handPos.getX(), handPos.getZ());
      }

      // turn the model using a circle gesture
      GestureList gestures = frame.gestures();
      for (Gesture gesture : gestures) {
        if (gesture.type() == Type.TYPE_CIRCLE) {
          CircleGesture circle = new CircleGesture(gesture);

          // calculate clock direction using the angle between circle normal and pointable
          double turnAngle = circle.pointable().direction().angleTo(circle.normal()) - Math.PI/4;
              // clockwise if angle is less than 90 degrees; i.e. turnAngle is negative
          if (turnAngle != 0) {
            boolean isClockWise = (turnAngle < 0);
            turnModel(isClockWise);
          }
          break;    // don't bother with other gestures once a circle has been processed
        }
      }
    }
    wakeupOn(timeOut);    // set wakeup condition again
  } // end of processStimulus()



  private void translateModel(float x, float z)
  // move sprite forward, back, left, or right
  {
    if (Math.abs(x) > MIN_X_DIST) {
      if (x > 0)
        sprite.moveBy(MOVE_AMT, 0);     // right step
      else if (x < 0)
        sprite.moveBy(-MOVE_AMT, 0);    // left step
    }

    if (Math.abs(z) > MIN_Z_DIST) {
      if (z > 0)
        sprite.moveBy(0, MOVE_AMT);    // forwards
      else if (z < 0)
        sprite.moveBy(0, -MOVE_AMT);   // backwards
    }
  }  // end of translateModel()




  private void turnModel(boolean isClockWise)
  // rotate the model left or right about the y-axis
  {
    if (isClockWise)
      sprite.doRotateY(-ROTATE_AMT);    // clockwise
    else
      sprite.doRotateY(ROTATE_AMT);    // counter-clockwise
  }  // end of turnModel()


} // end of ModelBehavior class
